@@ -4,19 +4,12 @@
 #' 
 #' @description This tool implements the (exact version of) pharmacophore kernel for 3D 
 #' structures of molecules (\cite{Mahe, 2006}).
+#'  
 #' 
-#' @usage sd2gram3Dpharma(sdFileName, sdFileName2 = "", chargesFileName = "", 
-#'		chargesFileName2 = "",  edgeKernelType = c("RBF", "triangular"), 
-#'		edgeKernelParameter = 1, atomKernelMatrix = "", flagRemoveH = FALSE, 
-#'		morganOrder = as.integer(0), morganChargesThreshold = 0, 
-#'		fileType = c("sd","genericsd","kcf"), silentMode = FALSE, 
-#'		returnNormalized = FALSE, moleculeNameProperty = "", moleculeNameProperty2 = "")
-#' 
-#' 
-#' @param sdFileName File containing the molecules. Must be in MDL file format
+#' @param sdf File containing the molecules. Must be in MDL file format
 #' (MOL and SDF files). For more information on the file format see 
 #' http://en.wikipedia.org/wiki/Chemical_table_file. Default = "missing".
-#' @param sdFileName2 A second file containing molecules. Must also be in SDF.
+#' @param sdf2 A second file containing molecules. Must also be in SDF.
 #' If specified the molecules of the first file will be compared with the 
 #' molecules of this second file. Default = "missing".
 #' @param chargesFileName A character with the name of the file containing
@@ -37,21 +30,16 @@
 #' types of atoms exist and consequently the more dissimilar will be the molecules.
 #' @param morganChargesThreshold specifies a threshold above which partial 
 #' Morgan charges are considered as positive/negative.
-#' @param fileType Which filetype was submitted.
 #' @param silentMode Whether or not the program should print progress reports
 #' to the standart output.
 #' @param returnNormalized A logical specifying whether a normalized kernel
 #' matrix should be returned. Default = TRUE.
-#' @param moleculeNameProperty A string which specifies the name of the property
-#' of the molecules in the sdFile from which the row names (and column names)
-#' are read from. Default = "".
-#' @param moleculeNameProperty2 A string which specifies the name of the property
-#' of the molecules in the sdFile 2 from which the column names are read from.
-#' Default = "".
+#' @param detectArom Whether aromatic rings should be detected and aromatic
+#' bonds should a special bond type. (Default = TRUE).
 #' @examples 
 #' sdfolder <- system.file("sample_data",package="Rchemcpp")
 #' sdf <- list.files(sdfolder,full.names=TRUE,pattern="small")
-#' K <- sd2gram(sdf, moleculeNameProperty="Compound Name")
+#' K <- sd2gram(sdf)
 #' @return A numeric matrix containing the similarity values between the
 #' molecules.
 #' @author Michael Mahr <rchemcpp@@bioinf.jku.at>
@@ -61,24 +49,16 @@
 #' with support vector machines. Technical Report, HAL:ccsd-00020066, Ecole des
 #' Mines de Paris, March 2006.
 #' 
-#' @examples 
-#' sdfolder <- system.file("sample_data",package="Rchemcpp")
-#' sdf <- list.files(sdfolder,full.names=TRUE,pattern="tiny")
-#' moleculeNames <- sd2gram3Dpharma(sdf)
-#' 
 #' @export
 
 
-sd2gram3Dpharma = function(sdFileName, sdFileName2 = "", chargesFileName = "", 
+sd2gram3Dpharma = function(sdf, sdf2, chargesFileName = "", 
 		chargesFileName2 = "",  edgeKernelType = c("RBF", "triangular"), 
 		edgeKernelParameter = 1, atomKernelMatrix = "", flagRemoveH = FALSE, 
-		morganOrder = as.integer(0), morganChargesThreshold = 0, 
-		fileType = c("sd","genericsd","kcf"), silentMode = FALSE, 
-		returnNormalized = FALSE, moleculeNameProperty = "", moleculeNameProperty2 = "")
+		morganOrder = as.integer(0), morganChargesThreshold = 0, silentMode = FALSE, 
+		returnNormalized = TRUE, detectArom=TRUE)
 {
 	
-	if(!is.character(sdFileName)) stop("sdFileName must be string")
-	if(!is.character(sdFileName2)) stop("sdFileName2 must be string")
 	if(!is.character(chargesFileName)) stop("chargesFileName must be string")
 	if(!is.character(chargesFileName2)) stop("chargesFileName2 must be string")
 	
@@ -91,49 +71,90 @@ sd2gram3Dpharma = function(sdFileName, sdFileName2 = "", chargesFileName = "",
 	morganOrder <- as.integer(morganOrder)
 	if(!is.numeric(morganChargesThreshold)) stop("morganchargesThreshold must be numeric")
 	
-	if(!is.character(fileType)) stop("fileType must be string")
 	if(!is.logical(silentMode)) stop("silentMode must be logical")
 	if(!is.logical(returnNormalized)) stop("returnNormalized must be logical")
 	
-	if (missing(fileType)) {fileType = fileType[1]}
-	fileType = match.arg(fileType)
-	
+
 	if (missing(edgeKernelType)) {edgeKernelType = edgeKernelType[1]}
 	edgeKernelType = match.arg(edgeKernelType)
 	
 	#--
 	
-	if((sdFileName2 == "") && (chargesFileName2 != ""))
+	if((missing(sdf2)) && (chargesFileName2 != ""))
 		print("chargesFilename2 is useless") 
 	
-	if((sdFileName2 != "") && (chargesFileName == "") && (chargesFileName2 != "")) 
+	if((missing(sdf2)) && (chargesFileName == "") && (chargesFileName2 != "")) 
 		stop("both chargesFileNames have to be specified") 
-	if((sdFileName2 != "") && (chargesFileName != "") && (chargesFileName2 == ""))
+	if((missing(sdf2)) && (chargesFileName != "") && (chargesFileName2 == ""))
 		stop("both chargesFileNames have to be specified") 
-	
-	
-	
 	if( atomKernelMatrix != "" ){
 		loadGramAtoms( atomKernelMatrix );	# atomKernelMatrix is set by the
 	}
 	
 	
-	if( sdFileName2 != "" ){ # i.e., test set version.
-		
-		aSet = new (Rchemcpp::Rmoleculeset);
-		aSet2 = new (Rchemcpp::Rmoleculeset);
-		
-		if( fileType == "sd" ){
-			aSet$addSD( sdFileName, FALSE );
-			aSet2$addSD( sdFileName2, FALSE );
-		}else if( fileType == "genericsd" ){
-			aSet$addSD( sdFileName, TRUE );
-			aSet2$addSD( sdFileName2, TRUE );
-		}else if( fileType == "kcf" ){
-			aSet$addKCF( sdFileName );
-			aSet2$addKCF( sdFileName2 );
+	if(inherits(sdf,"SDFset")){
+		aSet <- SDFsetToRmoleculeset(sdf,detectArom=detectArom)[[1]]
+		molnames <- ChemmineR::sdfid(sdf)
+		molnames2 <- molnames
+		if (!missing(sdf2)){
+			if (inherits(sdf2,"SDFset")){
+				aSet2 <- SDFsetToRmoleculeset(sdf2,detectArom=detectArom)[[1]]
+				molnames2 <- ChemmineR::sdfid(sdf2)
+			} else 
+				stop("Input must be existing SDF files or \"SDFset\" objects.")	
 		}
 		
+	} else if (inherits(sdf,"Rcpp_Rmoleculeset")) {
+		aSet <- sdf
+		molnames <- NULL
+		molnames2 <- NULL
+		if (!missing(sdf2)){
+			if (inherits(sdf2,"Rcpp_Rmoleculeset")){
+				aSet2 <- sdf2
+			} else 
+				stop("Input must be existing SDF files or \"SDFset\" objects.")
+		}
+		
+	} else if (is.character(sdf) & file.exists(sdf)) {
+		
+		TT <- try({
+					aSet <- new(Rmoleculeset)					
+					aSet$addSD(sdf,TRUE)
+					molnames <- getMoleculeNamesFromSDF(sdf)
+					molnames2 <- molnames
+					
+					if (!missing(sdf2)){
+						if (is.character(sdf2) & file.exists(sdf2)){
+							aSet2 <- new(Rmoleculeset)
+							aSet$addSD(sdf2,TRUE)
+							molnames2 <- getMoleculeNamesFromSDF(sdf2)		
+						} else 
+							stop("Input must be existing SDF files or \"SDFset\" objects.")	
+					} 
+				})
+		if (inherits(TT,"try-error")){
+			aSetList <- readRmoleculeset(sdf,detectArom=detectArom)
+			aSet <- aSetList[[1]]
+			molnames <- aSetList[[3]]
+			molnames2 <- aSetList[[3]]
+			if (!missing(sdf2)){
+				if (is.character(sdf2) & file.exists(sdf2)){
+					aSetList2  <-  readRmoleculeset(sdf2,detectArom=detectArom)
+					aSet2 <- aSetList2[[1]]
+					molnames2 <- aSetList2[[3]]
+				} else 
+					stop("Input must be existing SDF files or \"SDFset\" objects.")	
+			} 
+		}
+		
+	} else {
+		stop("Input must be existing SDF files or \"SDFset\" objects.")
+	}
+	
+	
+	if(!missing(sdf2)){ # i.e., test set version.
+		
+		
 		# read partial charges if specified on command line
 		if(chargesFileName != ""){
 			if( !silentMode ){
@@ -227,16 +248,6 @@ sd2gram3Dpharma = function(sdFileName, sdFileName2 = "", chargesFileName = "",
 	}
 	else{ # self-set version
 		
-		# create a new moleculeSet
-		aSet = new (Rchemcpp::Rmoleculeset);
-		
-		if( fileType == "sd" ){
-			aSet$addSD( sdFileName, FALSE );
-		}else if( fileType == "genericsd" ){
-			aSet$addSD( sdFileName, TRUE );
-		}else if( fileType == "kcf" ){
-			aSet$addKCF( sdFileName );
-		}
 		
 		# read partial charges if specified on command line
 		if(chargesFileName != ""){
@@ -326,50 +337,12 @@ sd2gram3Dpharma = function(sdFileName, sdFileName2 = "", chargesFileName = "",
 	}
 	
 	
-	#name the molecules
-	if (moleculeNameProperty != "")
-	{
-		molnames = c()
-		for (i in 0:(aSet$numMolecules() -1))
-		{
-			mol = aSet$getMolByIndex(i);
-			
-			if( moleculeNameProperty %in% mol$listStringDescriptors() )
-			{
-				molnames = c(molnames, mol$getStringDescriptorValue(moleculeNameProperty))
-			}
-			else
-			{
-				molnames = c(molnames, i)
-			}	
-		}
-		rownames(K) <- molnames
-
-		if (sdFileName2==""){
-			colnames(K) <- molnames
-		} else {
-			molnames2 = c()
-			for (i in 0:(aSet2$numMolecules() -1))
-			{
-				mol2 = aSet2$getMolByIndex(i);
-			
-				if( moleculeNameProperty2 %in% mol2$listStringDescriptors() )
-				{
-					molnames2 = c(molnames2, mol2$getStringDescriptorValue(moleculeNameProperty2))
-				}
-				else
-				{
-					molnames2 = c(molnames2, i)
-				}	
-			}
-			colnames(K) <- molnames2
-		}
-			
-	}
+	rownames(K) <- molnames
+	colnames(K) <- molnames2
+	
 	
 	return ( K )
 	
-	
 }
 
 
