@@ -3,18 +3,10 @@
 #' @description This function computes several walk-based graph kernel functions
 #' based on finite length walks and a fast implementation for input sd file(s).
 #'
-#' @usage sd2gramSpectrum(sdFileName, sdFileName2 = "", 
-#'		kernelType = c("spectrum", "tanimoto", "minmaxTanimoto","marginalized","lambda"), 
-#'		margKernelEndProbability = 0.1, lambdaKernelLambda = 1.0, 
-#'		depthMax = as.integer(0), onlyDepthMax = FALSE , flagRemoveH = FALSE, 
-#'		morganOrder = as.integer(0), fileType = c("sd","genericsd","kcf"), 
-#'		silentMode = FALSE, returnNormalized = FALSE, moleculeNameProperty = "",
-#'		moleculeNameProperty2 = "")
-#' 
-#' @param sdFileName File containing the molecules. Must be in MDL file format
+#' @param sdf File containing the molecules. Must be in MDL file format
 #' (MOL and SDF files). For more information on the file format see 
 #' http://en.wikipedia.org/wiki/Chemical_table_file. Default = "missing"
-#' @param sdFileName2 A second file containing molecules. Must also be in SDF.
+#' @param sdf2 A second file containing molecules. Must also be in SDF.
 #' If specified the molecules of the first file will be compared with the 
 #' molecules of this second file. Default = "missing".
 #' @param kernelType Sets which kernel is to be used. Options are "spectrum (Spectrum 
@@ -33,49 +25,36 @@
 #' @param morganOrder The order of the DeMorgan Indices to be used. If set to
 #' zero no DeMorgan Indices are used. The higher the order the more different
 #' types of atoms exist and consequently the more dissimilar will be the molecules.
-#' @param fileType Which filetype was submitted.
 #' @param silentMode Whether or not the program should print progress reports
 #' to the standart output.
 #' @param returnNormalized A logical specifying whether a normalized kernel
 #' matrix should be returned. Default = TRUE.
-#' @param moleculeNameProperty A string which specifies the name of the property
-#' of the molecules in the sdFile from which the row names (and column names)
-#' are read from. Default = "".
-#' @param moleculeNameProperty2 A string which specifies the name of the property
-#' of the molecules in the sdFile 2 from which the column names are read from.
-#' Default = "".
+#' @param detectArom Whether aromatic rings should be detected and aromatic
+#' bonds should a special bond type. (Default = TRUE).
 #' @examples 
 #' sdfolder <- system.file("sample_data",package="Rchemcpp")
 #' sdf <- list.files(sdfolder,full.names=TRUE,pattern="small")
-#' K <- sd2gramSpectrum(sdf, moleculeNameProperty="Compound Name")
+#' K <- sd2gramSpectrum(sdf)
 #' @return A numeric matrix containing the similarity values between the
 #' molecules.
 #' @author Michael Mahr <rchemcpp@@bioinf.jku.at>
 #' c++ function written by Jean-Luc Perret and Pierre Mahe
 #' 
-#' @examples 
-#' sdfolder <- system.file("sample_data",package="Rchemcpp")
-#' sdf <- list.files(sdfolder,full.names=TRUE,pattern="tiny")
-#' moleculeNames <- sd2gramSpectrum(sdf)
-#' 
 #' @export
 
 
-sd2gramSpectrum = function(sdFileName, sdFileName2 = "", 
+sd2gramSpectrum = function(sdf, sdf2, 
 		kernelType = c("spectrum", "tanimoto", "minmaxTanimoto","marginalized","lambda"), 
 		margKernelEndProbability = 0.1, lambdaKernelLambda = 1.0, 
-		depthMax = as.integer(0), onlyDepthMax = FALSE , flagRemoveH = FALSE, 
-		morganOrder = as.integer(0), fileType = c("sd","genericsd","kcf"), 
-		silentMode = FALSE, returnNormalized = FALSE, moleculeNameProperty = "",
-		moleculeNameProperty2 = "")
+		depthMax = as.integer(3), onlyDepthMax = FALSE , flagRemoveH = FALSE, 
+		morganOrder = as.integer(0), 
+		silentMode = FALSE, returnNormalized = TRUE,detectArom=TRUE)
 {
 	margKernelConvgce = 10000;
 	
 	margKernelSkipSkeleton = FALSE;
 	
-	if(!is.character(sdFileName)) stop("sdFileName must be string")
-	if(!is.character(sdFileName2)) stop("sdFileName2 must be string")
-	
+
 	if(!is.character(kernelType)) stop("kernelType must be a string")
 	if(!is.numeric(margKernelEndProbability)) stop("margKernelEndProbability must be numeric")
 	if(!is.numeric(lambdaKernelLambda)) stop("lambdaKernelLambda must be numeric")
@@ -87,14 +66,10 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 	if(!is.numeric(morganOrder)) stop("morganOrder must be integer")
 	morganOrder <- as.integer(morganOrder)
 	
-	if(!is.character(fileType)) stop("fileType must be string")
 	if(!is.logical(silentMode)) stop("silentMode must be logical")
 	if(!is.logical(returnNormalized)) stop("returnNormalized must be logical")
 	
-	
-	if (missing(fileType)) {fileType = fileType[1]}
-	fileType = match.arg(fileType)
-	
+
 	if (missing(kernelType)) {kernelType = kernelType[1]}
 	kernelType = match.arg(kernelType)
 	
@@ -110,31 +85,73 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 	kernelTypeIndex = as.integer(which(c("spectrum", "tanimoto", "minmaxTanimoto","marginalized","lambda") == kernelType)-1)
 	
 	
-	aSet = new (Rchemcpp::Rmoleculeset);
-	aSet2 = new (Rchemcpp::Rmoleculeset);
-	
+	if(inherits(sdf,"SDFset")){
+		aSet <- SDFsetToRmoleculeset(sdf,detectArom=detectArom)[[1]]
+		molnames <- ChemmineR::sdfid(sdf)
+		molnames2 <- molnames
+		if (!missing(sdf2)){
+			if (inherits(sdf2,"SDFset")){
+				aSet2 <- SDFsetToRmoleculeset(sdf2,detectArom=detectArom)[[1]]
+				molnames2 <- ChemmineR::sdfid(sdf2)
+			} else 
+				stop("Input must be existing SDF files or \"SDFset\" objects.")	
+		}
+		
+	} else if (inherits(sdf,"Rcpp_Rmoleculeset")) {
+		aSet <- sdf
+		molnames <- NULL
+		molnames2 <- NULL
+		if (!missing(sdf2)){
+			if (inherits(sdf2,"Rcpp_Rmoleculeset")){
+				aSet2 <- sdf2
+			} else 
+				stop("Input must be existing SDF files or \"SDFset\" objects.")
+		}
+		
+	} else if (is.character(sdf) & file.exists(sdf)) {
+		
+		TT <- try({
+					aSet <- new(Rmoleculeset)					
+					aSet$addSD(sdf,TRUE)
+					molnames <- getMoleculeNamesFromSDF(sdf)
+					molnames2 <- molnames
+					
+					if (!missing(sdf2)){
+						if (is.character(sdf2) & file.exists(sdf2)){
+							aSet2 <- new(Rmoleculeset)
+							aSet$addSD(sdf2,TRUE)
+							molnames2 <- getMoleculeNamesFromSDF(sdf2)		
+						} else 
+							stop("Input must be existing SDF files or \"SDFset\" objects.")	
+					} 
+				})
+		if (inherits(TT,"try-error")){
+			aSetList <- readRmoleculeset(sdf,detectArom=detectArom)
+			aSet <- aSetList[[1]]
+			molnames <- aSetList[[3]]
+			molnames2 <- aSetList[[3]]
+			if (!missing(sdf2)){
+				if (is.character(sdf2) & file.exists(sdf2)){
+					aSetList2  <-  readRmoleculeset(sdf2,detectArom=detectArom)
+					aSet2 <- aSetList2[[1]]
+					molnames2 <- aSetList2[[3]]
+				} else 
+					stop("Input must be existing SDF files or \"SDFset\" objects.")	
+			} 
+		}
+		
+	} else {
+		stop("Input must be existing SDF files or \"SDFset\" objects.")
+	}
 	
+	#browser()
 	# if sflag = 1 --> SEPARATE TEST SET
-	if( sdFileName2 != "" ){
+	if( !missing(sdf2) ){
 		
 		# 1 - data initialization
 		# -----------------------
 		# read the set of molecules
-		if( fileType == "sd" ){
-			aSet$addSD( sdFileName, FALSE );
-			aSet2$addSD( sdFileName2, FALSE );  
-		}else if( fileType == "genericsd" ){
-			aSet$addSD( sdFileName, TRUE );
-			aSet2$addSD( sdFileName2, TRUE );   
-		}else if( fileType == "kcf" ){
-			aSet$addKCF( sdFileName );
-			aSet2$addKCF( sdFileName2 );
-		}
-		if( !silentMode ){
-			print(paste("*** sd file added ; number of molecules in the set 1 =  ", aSet$numMolecules() ));
-			print(paste("*** sd file added ; number of molecules in the set 2 =  ", aSet2$numMolecules() ));
-		}
-		
+
 		# remove H when specified on command line
 		if( flagRemoveH == TRUE ){
 			if( !silentMode ){
@@ -172,6 +189,7 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 		# 3 - compute the gram matrix
 		# ----------------------------
 		gramSpectrum_test( aSet, depthMax, kernelTypeIndex, kernelParam, onlyDepthMax, silentMode);
+		
 		if( !silentMode ){
 			print("gramComputeSpectrum (test) OK");
 		}
@@ -215,16 +233,6 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 		# 1 - data initialization
 		# -----------------------
 		# read the set of molecules  
-		if( fileType == "sd" ){
-			aSet$addSD( sdFileName, FALSE );
-		}else if( fileType == "genericsd" ){
-			aSet$addSD( sdFileName, TRUE );
-		}else if( fileType == "kcf" ){
-			aSet$addKCF( sdFileName );
-		}
-		if( !silentMode ){
-			print (paste("*** sd file added ; number of molecules =  ", aSet$numMolecules() ));
-		}
 		
 		# remove H when specified on command line
 		if( flagRemoveH == TRUE ){
@@ -259,6 +267,7 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 		# 3 - compute the gram matrix
 		# ---------------------------
 		gramSpectrum_self( aSet, depthMax, kernelTypeIndex, kernelParam, onlyDepthMax, silentMode);
+		
 		if( !silentMode ){
 			print("gramComputeSpectrum (self) OK");
 		}
@@ -298,46 +307,9 @@ sd2gramSpectrum = function(sdFileName, sdFileName2 = "",
 	}  
 	
 	
-	#name the molecules
-	if (moleculeNameProperty != "")
-	{
-		molnames = c()
-		for (i in 0:(aSet$numMolecules() -1))
-		{
-			mol = aSet$getMolByIndex(i);
-			
-			if( moleculeNameProperty %in% mol$listStringDescriptors() )
-			{
-				molnames = c(molnames, mol$getStringDescriptorValue(moleculeNameProperty))
-			}
-			else
-			{
-				molnames = c(molnames, i)
-			}	
-		}
-		rownames(K) <- molnames
-
-		if (sdFileName2==""){
-			colnames(K) <- molnames
-		} else {
-			molnames2 = c()
-			for (i in 0:(aSet2$numMolecules() -1))
-			{
-				mol2 = aSet2$getMolByIndex(i);
-			
-				if( moleculeNameProperty2 %in% mol2$listStringDescriptors() )
-				{
-					molnames2 = c(molnames2, mol2$getStringDescriptorValue(moleculeNameProperty2))
-				}
-				else
-				{
-					molnames2 = c(molnames2, i)
-				}	
-			}
-			colnames(K) <- molnames2
-		}
-			
-	}
+	rownames(K) <- molnames
+	colnames(K) <- molnames2
+	
 	
 	return ( K )
 	
